import './spinner.css'
import { MovieWall } from './objects/movie-wall'
import { HamburgerMenu } from './objects/HamburgerMenu'
import { useState, useEffect } from 'react'
import tmdb from './objects/tmdb'
import { Sidebar } from './objects/sidebar'
import { Movie } from './objects/movie-wall'
import CrossfadeImage from 'react-crossfade-image'
import { WatchProviders } from './objects/whereToWatch'
import useWindowDimensions from './hooks/useWindowDimensions'

export type Config = {
  language: string,
  region: string,
  page: number,
  locale: string
}
class MovieClass implements Movie {
  id = 0
  poster_path = null
  backdrop_path = null
  title = null
  overview = null
  vote_average = 0
}

const splash: Movie = {
  id: 0,
  overview: "Select a movie to see others like it. Keep going until you find something to watch!",
  title: "Welcome to Movie Explorer!",
  vote_average: 0,
  poster_path: null,
  backdrop_path: null
}

class WatchProvidersClass implements WatchProviders {
  flatrate = [];
  rent = [];
  buy = [];
  link = "";
}


function App() {
  const [config, setConfig] = useState<Config>({
    language: "es",
    region: "spain",
    page: 1,
    locale: "ES"
  })
  const [movies, setMovies] = useState([new MovieClass()])
  const [backdrop, setBackdrop] = useState("")
  const [chosenMovie, setChosenMovie] = useState(splash)
  const [similarMoviesAvailable, setSimilarMoviesAvailable] = useState(true)
  const [watchProviders, setWatchProviders] = useState(new WatchProvidersClass())
  useEffect(() => { tmdb.getPopular(config, setMovies) }, [])
  useEffect(() => {
    const bgString = chosenMovie.backdrop_path ?
      tmdb.makeBgImgUrl(chosenMovie.backdrop_path)
      : tmdb.makeBgImgUrl(movies[0]?.backdrop_path ?? "")
    setBackdrop(bgString)
  }, [movies])
  useEffect(() => {
    tmdb.getWhereToWatch(chosenMovie, setWatchProviders, config)
  }, [chosenMovie])
  //Whenever config changes, refresh data for current selection
  useEffect(() => {
    console.log("CONFIG CHANGED!!")
    tmdb.getMovie(config, chosenMovie.id, setChosenMovie)
    tmdb.getSimilar(config, chosenMovie, setMovies, setSimilarMoviesAvailable)
  }, [config])
  const crossfadeImageStyles = {
    width: "100vw",
    height: "100vh",
    aspectRatio: "16/9",
    objectFit: "clip",
  }

  const { width } = useWindowDimensions()


  function trimMoviesForScreen(movies: Movie[]) {
    let desiredLength = 0
    if (width > 1280) {
      desiredLength = 18
    } else if (width > 400) {
      desiredLength = 16
    } else { desiredLength = 15 }
    //ensure movies.length is a multiple of 6 (so it makes an orphanless grid)
    while (movies.length < desiredLength && movies.length >= 6) { desiredLength = desiredLength - 6 }
    return movies.slice(0, desiredLength)
  }
  //TODO: search bar
  //TODO: genre search?
  return (
    <>

      <div className="w-screen h-screen flex flex-col absolute overflow-hidden">
        <div className="-z-10 bg-black/20 w-full h-full absolute"></div>
        <div className="-z-20 absolute h-full blur-xl bg-black">
          <CrossfadeImage src={backdrop} style={crossfadeImageStyles} />
        </div>
        <header className="bg-black py-2">
          <HamburgerMenu setConfig={setConfig} />
        </header>
        <main className="flex flex-col-reverse xl:flex-row justify-start sm:justify-center items-center sm:gap-8 grow">
          {/* //TODO: fix initial screen  */}
          <Sidebar className="flex flex-col justify-around sm:justify-between items-center bg-black/80 text-white w-full sm:w-96 h-64 sm:h-full" movie={chosenMovie} similarMoviesAvailable={similarMoviesAvailable} watchProviders={watchProviders} config={config} />
          <MovieWall
            className={"flex flex-wrap gap-2 flex-grow w-full md:w-11/12 justify-center items-center xl:max-w-5xl p-2"}
            movies={trimMoviesForScreen(movies)}
            setMovies={setMovies}
            config={config}
            setChosenMovie={setChosenMovie}
            setSimilarMoviesAvailable={setSimilarMoviesAvailable}
          />
        </main>
      </div>
    </>
  )
}

export default App
